import cv2
import argparse
import numpy as np

argparser = argparse.ArgumentParser()

argparser.add_argument('--img', type=str)
argparser.add_argument('--out', type=str)

args = argparser.parse_args()

confidence, threshold = 0.4, 0.3 

labelPath = './obj.names'
labels = open(labelPath).read().strip().split('\n')
weightsPath = './yolov3-tiny.weights'
configPath = './yolov3-tiny.cfg'
net = cv2.dnn.readNetFromDarknet(configPath, weightsPath)

layer_names = net.getLayerNames()
yolo_layers = []
for i in net.getUnconnectedOutLayers():
    yolo_layers.append(layer_names[i[0] - 1])

def draw_bb(img, boxes, confidences, classids, idxs, labels):
    if len(idxs):
        for i in idxs.flatten():
            x, y = boxes[i][0], boxes[i][1]
            w, h = boxes[i][2], boxes[i][3]

            cv2.rectangle(img, (x,y), (x+w, y+h), (0, 255, 0), 10)
            text = "{}:{:2.5f}".format(labels[classids[i]], confidences[i])
            cv2.putText(img, text, (x, y-10), cv2.FONT_HERSHEY_COMPLEX, 2, (255, 70, 0), 3)

    return img

    boxes, confidences, class_ids = [], [], []

def predict(net, layer_names, height, width, img, labels):

    blob = cv2.dnn.blobFromImage(img, 1/255.0, (416, 416), swapRB=True, crop=False)
    net.setInput(blob)
    layerOutputs = net.forward(layer_names)

    boxes, confidences, class_ids = [], [], []

    for out in layerOutputs:
        for detection in out:
            scores = detection[5:]
            class_id = np.argmax(scores)
            detect_confidence = scores[class_id]

            if detect_confidence > confidence:
                box = detection[0:4] * np.array([width, height, width, height])
                centerX, centerY, box_width, box_height = box.astype('int')

                x = int(centerX - (box_width / 2))
                y = int(centerY - (box_height / 2))

                boxes.append([x, y, int(box_width), int(box_height)])
                confidences.append(float(detect_confidence))
                class_ids.append(class_id)

    idxs = cv2.dnn.NMSBoxes(boxes, confidences, confidence, threshold)

    img = draw_bb(img, boxes, confidences, class_ids, idxs, labels)

    return img, boxes, confidences, class_ids, idxs

if args.img:
    img = cv2.imread(args.img)
    height, width = img.shape[:2]
    img, _, _, _, _ = predict(net, yolo_layers, height, width, img, labels)
    
    if args.out:
        cv2.imwrite(args.out, img)
    else:
        img = cv2.resize(img, (800, 800))
        cv2.imshow('Prediction', img)
        cv2.waitKey(0)
